# ArgParser.py
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# Copyright © 2012 Gwendal Le Bihan
# 
# This file is part of Torrent Search.
# 
# Torrent Search is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Torrent Search is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Torrent Search.  If not, see <http://www.gnu.org/licenses/>.

import argparse, gettext, os, logging
from ._settings import SETTINGS
from .env import DEFAULT_SHARE_PATH, APPDATA_PATH
from .informations import UNIX_APPNAME

DEBUG_LEVELS = {
   0: logging.FATAL,
   1: logging.ERROR,
   2: logging.WARNING,
   3: logging.INFO,
   4: logging.DEBUG
}

class ArgParser(argparse.ArgumentParser):
    def __init__(self):
        argparse.ArgumentParser.__init__(self, add_help = False)
        
        self.add_argument('-h',  '--help', action='store_true', dest='show_help', help = 'show this help message and exit')
        self.add_argument('--share-dir', dest='share_dir', default = DEFAULT_SHARE_PATH)
        
        for section_title, section_data in SETTINGS:
            for setting in section_data:
                if setting['allow_command_line']:
                    long_key = '--' + setting['key'].replace('_', '-')
                    params = {'dest': setting['key']}
                    if 'description' in setting:
                        params['help'] = setting['description']
                    if setting['type'] == 'int':
                        params['type'] = int
                        if 'default' in setting:
                            params['default'] = int(setting['default'])
                    elif setting['type'] == 'bool':
                        if 'default' in setting:
                            default_value = (setting['default'].lower() == 'true')
                        else:
                            default_value = False
                        params['default'] = default_value
                        if default_value:
                            params['action'] = 'store_false'
                        else:
                            params['action'] = 'store_true'
                    elif setting['type'] in ['str_choice', 'str']:
                        if 'default' in setting:
                            params['default'] = setting['default']
                    else:
                        params = {}
                        print(long_key+':'+setting['type'])
                    if params:
                        if 'short_key' in setting and type(setting['short_key']) == str and len(setting['short_key']) == 1:
                            self.add_argument("-" + setting['short_key'], long_key, **params)
                        else:
                            self.add_argument(long_key, **params)
                            
    def parse_args(self):
        res = argparse.ArgumentParser.parse_args(self)
        gettext.install(UNIX_APPNAME, os.path.join(res.share_dir, "locale"))
        if res.debug_level > 4:
            res.debug_level = 4
        if res.debug_level < 0:
            res.debug_level = 0
        logging.getLogger().setLevel(DEBUG_LEVELS[res.debug_level])
        if res.show_help:
            self.print_help()
            exit(0)
        if res.plugins_path == "":
            res.plugins_path = os.path.join(APPDATA_PATH, "plugins")
        else:
            res.plugins_path = os.path.abspath(res.plugins_path)
        return res
