# HTTPManager.py
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# Copyright © 2012 Gwendal Le Bihan
# 
# This file is part of Torrent Search.
# 
# Torrent Search is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Torrent Search is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Torrent Search.  If not, see <http://www.gnu.org/licenses/>.

import httplib2, logging, threading, time

HTTP_MANAGER_NB_CONNECTIONS = 10
HTTP_MANAGER_CONNECTION_WAIT_DELAY_MS = 100

class _AsyncRequest(threading.Thread):
    def __init__(self, manager, uri, method='GET', body=None, headers=None, redirections=5, connection_type=None):
        threading.Thread.__init__(self, None, self._perform, None, (uri, method, body, headers, redirections, connection_type))
        
        self._manager = manager
        self._response = None
    
    def _perform(self, uri, method, body, headers, redirections, connection_type):
        self._response = self._manager.request(uri, method, body, headers, redirections, connection_type)
    
    def getResponse(self):
        return self._response

class HTTPManager(object):
    def __init__(self, application):
        self._application = application
        self._async_request_id = 0
        self._waiting_connections = []
        self._busy_connections = []
        self._stopped = False
        self._stopped_lock = threading.Lock()
        for i in range(HTTP_MANAGER_NB_CONNECTIONS):
            self._waiting_connections.append(httplib2.Http())
        
        self._connections_lock = threading.Lock()
    
    def request_async(self, uri, method='GET', body=None, headers=None, redirections=5, connection_type=None, callback = None):
        """
            Perform an asynchronous request
            Must be called from the main thread
        """
        if self.stopped:
            return
            
        self._async_request_id += 1
        request = _AsyncRequest(self, uri, method, body, headers, redirections, connection_type)
        self._application.wait_for_thread(request, self._on_async_request_completed, callback)
        request.start()
    
    def _on_async_request_completed(self, request, callback):
        if callback and callable(callback):
            callback(request.getResponse())
    
    def request(self, uri, method='GET', body=None, headers=None, redirections=5, connection_type=None):
        if self.stopped:
            return
            
        logging.debug("Initializing request on %s" % uri)
        
        connection = None
        while connection == None:
            self._connections_lock.acquire()
            if len(self._waiting_connections) > 0:
                logging.debug("Found free connection for request on %s" % uri)
                connection = self._waiting_connections[0]
                self._waiting_connections = self._waiting_connections[1:]
                self._busy_connections.append(connection)
            else:
                logging.debug("Did not find free connection for request on %s, will try again" % uri)
            self._connections_lock.release()
            if connection == None:
                time.sleep(HTTP_MANAGER_CONNECTION_WAIT_DELAY_MS / 100.)
            
        if not self.stopped:
            logging.debug("Performing HTTP request on %s" % uri)
            res = connection.request(uri, method, body, headers, redirections, connection_type)
        else:
            res = None
            
        logging.debug("Response received from %s, freeing connection" % uri)
        self._connections_lock.acquire()
        i = self._busy_connections.index(connection)
        del self._busy_connections[i]
        self._waiting_connections.append(connection)
        self._connections_lock.release()
        
        logging.debug("End of request on %s" % uri)
        
        return res
    
    def _get_stopped(self):
        self._stopped_lock.acquire()
        res = self._stopped
        self._stopped_lock.release()
        return res
    def _set_stopped(self, value):
        self._stopped_lock.acquire()
        self._stopped = value
        self._stopped_lock.release()
    stopped = property(_get_stopped, _set_stopped)
    
    def stop(self):
        self.stopped = True
