# SearchEngine.py
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# Copyright © 2012 Gwendal Le Bihan
# 
# This file is part of Torrent Search.
# 
# Torrent Search is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Torrent Search is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Torrent Search.  If not, see <http://www.gnu.org/licenses/>.


from .EventsObject import EventsObject
import os, logging, sys, lxml.etree, imp, threading, datetime

class PluginMetadata(object):
    def __init__(self, filename):
        self._metadata_filename = filename
        self._data = self._parse(filename)
    
    def _parse(self, filename):
        f = open(filename)
        rootNode = lxml.etree.fromstring(f.read())
        f.close()
        
        res = {}
        
        res["plugin_id"] = rootNode.get("id")
        
        for child in rootNode:
            res[child.tag] = child.text
        
        return res
    
    def _get_filename(self):
        return self._data["filename"]
    filename = property(_get_filename)
    
    def _get_classname(self):
        return self._data["classname"]
    classname = property(_get_classname)
    
    def _get_name(self):
        return self._data["name"]
    name = property(_get_name)
    
    def _get_plugin_id(self):
        return self._data["plugin_id"]
    plugin_id = property(_get_plugin_id)
    
    def _get_enabled(self):
        if "enabled" in self._data.keys():
            return (self._data["enabled"].lower() == "true")
        elif "default_enabled" in self._data.keys():
            return (self._data["default_enabled"].lower() == "true")
        else:
            return False
    def _set_enabled(self, value):
        if value != self.enabled:
            if value:
                self._data["enabled"] = "true"
            else:
                self._data["enabled"] = "false"
            self._save()
    enabled = property(_get_enabled, _set_enabled)
    
    def _format_xml(self):
        if self.enabled:
            enabled = "true"
        else:
            enabled = "false"
        return """<?xml version="1.0"?>
<plugin id="%s">
    <name>%s</name>
    <filename>%s</filename>
    <classname>%s</classname>
    <default_enabled>%s</default_enabled>
    <enabled>%s</enabled>
    <author>%s</author>
    <author_email>%s</author_email>
    <version>%s</version>
</plugin>""" % (self.plugin_id, self.name, self.filename, self.classname, self._data["default_enabled"], enabled, self._data["author"], self._data["author_email"], self._data["version"])
    
    def _save(self):
        xml_data = self._format_xml()
        f = open(self._metadata_filename, "w")
        f.write(xml_data)
        f.close()

class SearchResultError(Exception):
    pass

class SearchResult(EventsObject):
    def __init__(self, application, plugin, **data):
        EventsObject.__init__(self)
        
        self.plugin = plugin
        self._data = data
        self._application = application
        
        self._additional_infos_loaded = False
        self._additional_infos_loaded_lock = threading.Lock()
        
        if not "title" in data.keys() or type(data["title"]) != str or data["title"] == "":
            raise SearchResultError("Missing or incorrect \"title\" field")
    
    def _get_should_show(self):
        return self._application.should_show_result(self)
    should_show = property(_get_should_show)
    
    def _get_title(self):
        return self._data["title"].rstrip().lstrip().replace("\n", " ")
    title = property(_get_title)
    
    def _get_size(self):
        if "size" in self._data.keys() and type(self._data["size"] in [int, float]):
            return self._data["size"]
        else:
            return -1
    size = property(_get_size)
    def _get_formatted_size(self):
        if self.size >= 0:
            units = ["B", "KB", "MB", "GB", "TB"]
            value = self.size
            ui = 0
            while ui < len(units) - 1 and value >= 1024:
                ui += 1
                value /= 1024.
            return "%.2f %s" % (value, units[ui])
        else:
            return ""
    formatted_size = property(_get_formatted_size)
    
    def _get_date(self):
        if "date" in self._data.keys() and type(self._data["date"]) == datetime.date:
            return self._data["date"]
        else:
            return None
    date = property(_get_date)
    def _get_formatted_date(self):
        if self.date != None:
            return self.date.strftime("%Y-%m-%d")
        else:
            return ""
    formatted_date = property(_get_formatted_date)
    
    def _get_seeders(self):
        if "seeders" in self._data.keys() and type(self._data["seeders"] == int):
            return self._data["seeders"]
        else:
            return -1
    seeders = property(_get_seeders)
    def _get_formatted_seeders(self):
        if self.seeders >= 0:
            return str(self.seeders)
        else:
            return ""
    formatted_seeders = property(_get_formatted_seeders)
    
    def _get_leechers(self):
        if "leechers" in self._data.keys() and type(self._data["leechers"] == int):
            return self._data["leechers"]
        else:
            return -1
    leechers = property(_get_leechers)
    def _get_formatted_leechers(self):
        if self.leechers >= 0:
            return str(self.leechers)
        else:
            return ""
    formatted_leechers = property(_get_formatted_leechers)
    
    def _get_nb_comments(self):
        if "nb_comments" in self._data.keys() and type(self._data["nb_comments"] == int):
            return self._data["nb_comments"]
        else:
            return 0
    nb_comments = property(_get_nb_comments)
    def _get_formatted_nb_comments(self):
        if self.nb_comments > 0:
            return str(self.nb_comments)
        else:
            return ""
    formatted_nb_comments = property(_get_formatted_nb_comments)
    
    def _get_need_additional_infos(self):
        for key in ["title", "date", "size", "seeders", "leechers"]:
            if not key in self._data or not self._data[key]:
                return True
        if (not "download_url" in self._data.keys() or type(self._data["download_url"]) != str or self._data["download_url"] == "") and (not "magnet_link" in self._data.keys() or type(self._data["magnet_link"]) != str or self._data["magnet_link"] == ""):
            return True
        return False
    need_additional_infos = property(_get_need_additional_infos)
    
    def fetch_additional_infos(self):
        if "additional_infos_url" in self._data and type(self._data["additional_infos_url"] == str) and self._data["additional_infos_url"] != "":
            thread = threading.Thread(None, self._do_fetch_additional_infos, None, (self._data["additional_infos_url"],))
            self._application.timeout_add(100, self._check_additional_infos_loaded)
            thread.start()
    
    def _get_additional_infos_loaded(self):
        self._additional_infos_loaded_lock.acquire()
        res = self._additional_infos_loaded
        self._additional_infos_loaded_lock.release()
        return res
    def _set_additional_infos_loaded(self, value):
        self._additional_infos_loaded_lock.acquire()
        self._additional_infos_loaded = value
        self._additional_infos_loaded_lock.release()
    additional_infos_loaded = property(_get_additional_infos_loaded, _set_additional_infos_loaded)
        
    def _check_additional_infos_loaded(self):
        if self.additional_infos_loaded:
            self._trigger_event("additional_infos_loaded")
            return False
        else:
            return True
    
    def _do_fetch_additional_infos(self, url):
        try:
            data = self.plugin.retrieve_additional_infos(url)
            if type(data) != dict:
                data = {}
        except:
            data = {}
        
        for i in data:
            self._data[i] = data[i]
        
        self.additional_infos_loaded = True

class SearchEngine(EventsObject):
    def __init__(self, application, plugins_dir):
        EventsObject.__init__(self)
        
        self._request_id = 0
        self._plugins_dir = plugins_dir
        self._application = application
        
        self._plugins = []
    
    def _get_overall_progress(self):
        total_results = 0.
        valid_results = 0.
        for i in self._plugins:
            if i.results_count:
                total_results += i.results_count
                valid_results += i.valid_results_count
        if total_results > 0:
            return max(0, min(1, valid_results / total_results))
        else:
            return 0
    overall_progress = property(_get_overall_progress)
    
    def load_plugins(self):
        for i in self._plugins:
            i.destroy()
        self._plugins = []
        if os.path.exists(self._plugins_dir) and os.path.isdir(self._plugins_dir):
            for d in os.listdir(self._plugins_dir):
                if os.path.isdir(os.path.join(self._plugins_dir, d)):
                    self._load_plugin_from_path(os.path.join(self._plugins_dir, d))
        for i in self._plugins:
            i.connect_event("result_added", self._on_plugin_result_added)
            i.connect_event("search_finished", self._on_plugin_search_finished)
        self._trigger_event("plugins_loaded", self._plugins)
    
    def _load_plugin_from_path(self, path):
        logging.debug("Loading plugin from \"%s\"" % path)
        
        try:
            metadata_file = os.path.join(path, "metadata.xml")
            if not os.path.exists(metadata_file) or not os.path.isfile(metadata_file):
                logging.warn("Metadata file missing, aborting")
                return
                
            try:
                metadata = PluginMetadata(metadata_file)
            except:
                exc_class, exc_instance, traceback = sys.exc_info()
                logging.warn("Failed to load metadata file : %s" % str(exc_instance))
                return
                
            plugin_file = os.path.join(path, metadata.filename)
            if not os.path.exists(plugin_file) or not os.path.isfile(plugin_file):
                logging.warn("Plugin file missing, aborting")
                return
            
            f = open(plugin_file)
            m = imp.load_module(metadata.plugin_id, f, metadata.filename, ('.py', 'r', imp.PY_SOURCE))
            plugin_class = getattr(m, metadata.classname)
            plugin = plugin_class(self._application, metadata)
            if self._application.get_setting("debug_plugin") == "" or self._application.get_setting("debug_plugin") == plugin.plugin_id:
                self._plugins.append(plugin)
        except:
            exc_class, exc_instance, traceback = sys.exc_info()
            logging.warn("Error while loading plugin : %s" % str(exc_instance))
    
    def _on_plugin_result_added(self, plugin, request_id, result):
        if request_id == self._request_id:
            try:
                search_result = SearchResult(self._application, plugin, **result)
                plugin.valid_results_count += 1
                self._trigger_event("result_added", plugin, search_result)
                self._trigger_event("progress")
            except:
                exc_class, exc_instance, traceback = sys.exc_info()
                logging.warn("Could not load result coming from plugin \"%s\" : %s" % (plugin.plugin_id, str(exc_instance)))
    
    def _on_plugin_search_finished(self, plugin, request_id):
        self._trigger_event("progress")
    
    def _get_enabled_plugins(self):
        res = []
        for i in self._plugins:
            if i.enabled:
                res.append(i)
        return res
    enabled_plugins = property(_get_enabled_plugins)
    
    def _get_plugins(self):
        return self._plugins
    plugins = property(_get_plugins)
    
    def _get_search_finished(self):
        for i in self._plugins:
            if not i.search_finished:
                return False
        return True
    search_finished = property(_get_search_finished)
    
    def stop(self):
        self._request_id += 1
        for i in self._plugins:
            i.stop()
            i.results_count = i.valid_results_count
        self._trigger_event("progress")
    
    def run(self, pattern):
        self._request_id += 1
        for i in self.enabled_plugins:
            i.run(pattern, self._request_id)
