# SearchPlugin.py
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# Copyright © 2012 Gwendal Le Bihan
# 
# This file is part of Torrent Search.
# 
# Torrent Search is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Torrent Search is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Torrent Search.  If not, see <http://www.gnu.org/licenses/>.

from .EventsObject import EventsObject
import threading, logging
from .HTMLTools import HTMLDocument

class SearchThread(threading.Thread):
    def __init__(self, plugin, request_id, search_function, pattern):
        threading.Thread.__init__(self, None, search_function, None, (pattern,))
        self.request_id = request_id
        self._plugin = plugin
        self._must_stop = threading.Event()
        self.hasStopped = threading.Event()
    
    def stop(self):
        self._must_stop.set()
        self.join(0.1)
        return self.hasStopped.isSet()

class SearchPlugin(EventsObject):
    def __init__(self, application, metadata):
        EventsObject.__init__(self)
        
        self._application = application
        self._metadata = metadata
        
        self._search_thread = None
        self._results_queue = []
        self._results_queue_lock = threading.Lock()
        
        self._results_count = None
        self._valid_results_count = 0
        self._search_finished = True
        self._results_count_lock = threading.Lock()
        self._valid_results_count_lock = threading.Lock()
        self._search_finished_lock = threading.Lock()
    
    def run(self, pattern, request_id):
        self._search_thread = SearchThread(self, request_id, self._do_run, pattern)
        self._search_thread.start()
        self._application.timeout_add(100, self._check_progress)
    
    def _check_progress(self):
        if self._search_thread != None:
            self._results_queue_lock.acquire()
            while len(self._results_queue) > 0:
                self._trigger_event("result_added", self._search_thread.request_id, self._results_queue[0])
                del self._results_queue[0]
            self._results_queue_lock.release()
        if self._search_thread != None and not self._search_thread.hasStopped.isSet():
            return True
        if self._search_thread != None:
            self.results_count = self.valid_results_count
            logging.debug("Plugin \"%s\" finished search for request #%d" % (self.plugin_id, self._search_thread.request_id))
            self._trigger_event("search_finished", self._search_thread.request_id)
        return False
    
    def stop(self):
        if self._search_thread != None and not self._search_thread.hasStopped.isSet():
            logging.debug("Requesting plugin \"%s\" to stop" % self.plugin_id)
            if self._search_thread.stop():
                logging.debug("Plugin \"%s\" stopped" % self.plugin_id)
            else:
                logging.warn("Stopping plugin \"%s\" timed out, letting it stop on its own" % self.plugin_id)
        self._search_thread = None
    
    def _do_run(self, pattern):
        self.valid_results_count = 0
        self.results_count = None
        self.search_finished = False
        try:
            self._process_search(pattern)
        except:
            pass
        self.search_finished = True
        threading.currentThread().hasStopped.set()
    
    def must_stop(self):
        return threading.currentThread()._must_stop.isSet()
    
    def _get_plugin_id(self):
        return self._metadata.plugin_id
    plugin_id = property(_get_plugin_id)
    
    def _get_name(self):
        return self._metadata.name
    name = property(_get_name)
    
    def _get_enabled(self):
        return self._metadata.enabled
    def _set_enabled(self, value):
        self._metadata.enabled = value
    enabled = property(_get_enabled, _set_enabled)

    def _http_request(self, uri, method='GET', body=None, headers=None, redirections=5, connection_type=None):
        return self._application.http_manager.request(uri, method, body, headers, redirections, connection_type)
    
    def _parse_html(self, html_content):
        return HTMLDocument(html_content)
    
    def _warn(self, mesg):
        logging.warn(mesg)
    
    def _convert_size(self, value, unit):
        units = ['B', 'KB', 'MB', 'GB', 'TB']
        ui = units.index(unit.upper())
        res = value
        while ui > 0:
            res *= 1024
            ui -= 1
        return res
    
    def _add_result(self, **kwargs):
        self._results_queue_lock.acquire()
        self._results_queue.append(kwargs)
        self._results_queue_lock.release()
    
    def _get_valid_results_count(self):
        self._valid_results_count_lock.acquire()
        res = self._valid_results_count
        self._valid_results_count_lock.release()
        return res
    def _set_valid_results_count(self, value):
        self._valid_results_count_lock.acquire()
        self._valid_results_count = value
        self._valid_results_count_lock.release()
    valid_results_count = property(_get_valid_results_count, _set_valid_results_count)
    
    def _get_search_finished(self):
        self._search_finished_lock.acquire()
        res = self._search_finished
        self._search_finished_lock.release()
        return res
    def _set_search_finished(self, value):
        self._search_finished_lock.acquire()
        self._search_finished = value
        self._search_finished_lock.release()
    search_finished = property(_get_search_finished, _set_search_finished)
    
    def _get_results_count(self):
        self._results_count_lock.acquire()
        res = self._results_count
        self._results_count_lock.release()
        return res
    def _set_results_count(self, value):
        self._results_count_lock.acquire()
        self._results_count = value
        self._results_count_lock.release()
    results_count = property(_get_results_count, _set_results_count)
    
    def retrieve_additional_infos(self, url):
        return {}
