# SettingsManager.py
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# Copyright © 2012 Gwendal Le Bihan
# 
# This file is part of Torrent Search.
# 
# Torrent Search is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Torrent Search is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Torrent Search.  If not, see <http://www.gnu.org/licenses/>.

from .EventsObject import EventsObject
from ._settings import SETTINGS
from .env import APPDATA_PATH
from .fsutils import rec_mkdir
from .informations import UNIX_APPNAME
import logging, os, sys

class BaseSetting(EventsObject):
    def __init__(self, **kwargs):
        EventsObject.__init__(self)
        
        self._data = kwargs
        
        self._key = kwargs["key"]
        
        if "default" in kwargs.keys():
            self._value = self._parse_value(kwargs["default"])
        else:
            self._value = self._parse_value(None)
    
    def get_value(self):
        return self._value
    
    def set_value(self, value):
        new_value = self._parse_value(value)
        if new_value != self._value:
            self._value = new_value
            self._trigger_event("value_changed", new_value)
    
    def get_type(self):
        return self._data["type"]
    
    def __repr__(self):
        return "<%s key=%s value=%s>" % (self.__class__.__name__, self._key, str(self._value))
    
    def must_save(self):
        return (self._data["save"] == True)
    
    def format_save(self):
        res = "# %s : %s\n" % (self._key, self._data["name"])
        if self._data["description"]:
            res += "# %s\n" % self._data["description"]
        if "default" in self._data.keys():
            res += "# Default value : %s\n" % self._data["default"]
        res += "%s=%s\n\n" % (self._key, str(self._value))
        return res

class ChoiceSetting(BaseSetting):
    def _get_values(self):
        res = []
        for i in self._data["values"]:
            res.append({'key': self._parse_value(i['key']), 'label': i['label']})
        return res
    values = property(_get_values)

class IntSetting(BaseSetting):
    def _parse_value(self, value):
        if type(value) == int:
            res = value
        else:
            try:
                res = int(value)
            except:
                logging.warn("Incorrect integer value for setting \"%s\" : %s" % (self._key, str(value)))
                if not "default" in self._data.keys() or value == self._data["default"]:
                    res = 0
                else:
                    res = self._parse_value(self._data["default"])
        if "min_value" in self._data.keys() and res < self._data["min_value"]:
            logging.warn("Value out of range for setting \"%s\" : %d" % (self._key, res))
            res = self._data["min_value"]
        if "max_value" in self._data.keys() and res > self._data["max_value"]:
            logging.warn("Value out of range for setting \"%s\" : %d" % (self._key, res))
            res = self._data["max_value"]
        return res

class StrSetting(BaseSetting):
    def _parse_value(self, value):
        if type(value) == str:
            return value
        else:
            logging.warn("Incorrect string value for setting \"%s\" : %s" % (self._key, str(value)))
            return ""
    def __repr__(self):
        return "<%s key=%s value=\"%s\">" % (self.__class__.__name__, self._key, str(self._value))

class StrChoiceSetting(StrSetting, ChoiceSetting):
    def _parse_value(self, value):
        res = StrSetting._parse_value(self, value)
        for i in self._data["values"]:
            if res == i["key"]:
                return res
        logging.warn("Incorrect value for setting \"%s\" : %s (not in values list)" % (self._key, str(value)))
        if not "default" in self._data.keys() or value == self._data["default"]:
            if len(self._data["values"]) > 0:
                return self._data["values"][0]["key"]
            else:
                return ""
        else:
            return self._parse_value(self._data["default"])

class BoolSetting(BaseSetting):
    def _parse_value(self, value):
        if type(value) == bool:
            return value
        try:
            return (value.lower() == "true")
        except:
            logging.warn("Incorrect boolean value for setting \"%s\" : %s" % (self._key, str(value)))
            if not "default" in self._data.keys() or value == self._data["default"]:
                return False
            else:
                return self._parse_value(self._data["default"])

SETTINGS_TYPE_MAP = {
    "bool": BoolSetting,
    "int": IntSetting,
    "str": StrSetting,
    "str_choice": StrChoiceSetting
}

class SettingsManager(EventsObject):
    def __init__(self, cli_args):
        EventsObject.__init__(self)
        
        self._loading = True
        
        self._sections = []
        self._settings = {}
        
        for section_label, section_settings in SETTINGS:
            current_section = {"label": _(section_label), "settings": []}
            for setting_data in section_settings:
                if setting_data["type"] in SETTINGS_TYPE_MAP.keys():
                    setting = SETTINGS_TYPE_MAP[setting_data["type"]](**setting_data)
                    self._settings[setting_data["key"]] = setting
                    current_section["settings"].append(setting)
                    setting.connect_event("value_changed", self._on_setting_value_changed, setting_data["key"])
                else:
                    logging.warn("Unsupported setting type \"%s\"" % setting_data["type"])
        
        self._no_save = False
        
        if not os.path.exists(APPDATA_PATH):
            logging.debug("Creating configuration folder")
            try:
                rec_mkdir(APPDATA_PATH)
            except:
                pass
        
        if not os.path.isdir(APPDATA_PATH):
            logging.warn("Could not create folder \"%s\", settings won't be saved" % APPDATA_PATH)
            self._no_save = True
        
        self._settings_filename = os.path.join(APPDATA_PATH, UNIX_APPNAME + ".cfg")
        if os.path.exists(self._settings_filename):
            if os.access(self._settings_filename, os.R_OK):
                self._load()
            else:
                logging.warn("Can't read \"%s\", permission denied" % self._settings_filename)
                self._no_save = True
            self._loading = False
        else:
            logging.debug("Configuration file does not exist, creating it now")
            self._loading = False
            self._save()
        
        for i in self._settings:
            if hasattr(cli_args, i):
                self.set_setting_value(i, getattr(cli_args, i))
    
    def _load(self):
        logging.debug("Loading settings from %s" % self._settings_filename)
        try:
            f = open(self._settings_filename)
            data = f.read()
            f.close()
            
            for line in data.splitlines():
                line = line.rstrip().lstrip().split("#")[0]
                if line != "" and "=" in line:
                    key, value = line.split("=", 2)
                    self.set_setting_value(key, value)
        except:
            exc_class, exc_instance, traceback = sys.exc_info()
            logging.warn("Could not load settings : %s" % str(exc_instance))
    
    def _save(self):
        if self._no_save or self._loading:
            return
        try:
            if os.path.exists(self._settings_filename) and not os.access(self._settings_filename, os.W_OK):
                logging.warn("Can't write \"%s\", permission denied, settings won't be saved" % self._settings_filename)
            else:
                data = ""
                for key in self._settings:
                    setting = self._settings[key]
                    if setting.must_save():
                        data += setting.format_save()
                f = open(self._settings_filename, "w")
                f.write(data)
                f.close()
        except:
            exc_class, exc_instance, traceback = sys.exc_info()
            logging.warn("Can't write settings file : %s" % str(exc_instance))
    
    def _on_setting_value_changed(self, setting, value, key):
        if not self._loading:
            self._trigger_event("setting_value_changed", key, value)
            logging.debug("Setting value changed, %s = %s" % (key, value))
            if setting.must_save():
                self._save()
    
    def get_setting(self, key):
        if key in self._settings.keys():
            return self._settings[key]
        else:
            logging.warn("Setting key \"%s\" does not exist" % key)
            return None
    
    def get_setting_value(self, key):
        setting = self.get_setting(key)
        if setting != None:
            return setting.get_value()
        
    def set_setting_value(self, key, value):
        setting = self.get_setting(key)
        if setting != None:
            setting.set_value(value)
