# frontend.py
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# Copyright © 2012 Gwendal Le Bihan
# 
# This file is part of Torrent Search.
# 
# Torrent Search is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Torrent Search is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Torrent Search.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import Gtk, Gdk, GObject
from TorrentSearch.informations import *
import os, logging
from .ResultsList import ResultsList
from .PreferencesPluginsList import PreferencesPluginsList
from TorrentSearch.EventsObject import EventsObject

class Gtk3Frontend(EventsObject):
    def __init__(self, application, share_path):
        EventsObject.__init__(self)
        
        self._application = application
        self._share_path = share_path
        self._window = None
        self._window_mapped_once = False
        self._ui_path = os.path.join(share_path, UNIX_APPNAME, "ui", "gtk3")
        application.search_engine.connect_event("progress", self._on_search_progress)
    
    def _on_search_progress(self, search_engine):
        self.show_search_progress(search_engine.overall_progress)
        self._stop_search_action.set_sensitive(not search_engine.search_finished)
    
    def timeout_add(self, delay, callback, *args):
        GObject.timeout_add(delay, callback, *args)
    
    def init(self):
        GObject.threads_init()
        self._build()
        self._window.show_all()
    
    def _display_about(self):
        self._about_dialog.show_all()
        self._about_dialog.run()
        self._about_dialog.hide()
    
    def _display_preferences(self):
        self._preferences_dialog.show_all()
        self._preferences_dialog.run()
        self._preferences_dialog.hide()
    
    def _build(self):
        builder = Gtk.Builder()
        builder.add_from_file(os.path.join(self._ui_path, UNIX_APPNAME + ".ui"))
        self._window = builder.get_object("main_window")
        self._about_dialog = builder.get_object("about_dialog")
        self._preferences_dialog = builder.get_object("preferences_dialog")
        
        self._window.connect("delete_event", lambda w,e: self.stop_mainloop())
        builder.get_object("quit_action").connect("activate", lambda a: self.stop_mainloop())
        builder.get_object("display_preferences_action").connect("activate", lambda a: self._display_preferences())
        builder.get_object("display_about_action").connect("activate", lambda a: self._display_about())
        
        builder.get_object("settings_min_size").set_range(0, 999)
        builder.get_object("settings_min_size").set_increments(10, 100)
        builder.get_object("settings_max_size").set_range(0, 999)
        builder.get_object("settings_max_size").set_increments(10, 100)
        
        self._bind_setting_widget(builder.get_object("settings_hide_no_seeders"), "hide_no_seeders")
        self._bind_setting_widget(builder.get_object("settings_filter_duplicates"), "filter_duplicates")
        self._bind_setting_widget(builder.get_object("settings_exact_phrase"), "exact_phrase")
        self._bind_setting_widget(builder.get_object("settings_all_words"), "all_words")
        self._bind_setting_widget(builder.get_object("settings_min_size_enabled"), "min_size_enabled")
        self._bind_setting_widget(builder.get_object("settings_max_size_enabled"), "max_size_enabled")
        self._bind_setting_widget(builder.get_object("settings_min_size"), "min_size")
        self._bind_setting_widget(builder.get_object("settings_max_size"), "max_size")
        self._bind_setting_widget(builder.get_object("settings_min_size_unit"), "min_size_unit")
        self._bind_setting_widget(builder.get_object("settings_max_size_unit"), "max_size_unit")
        self._bind_setting_widget(builder.get_object("settings_name_contains"), "name_contains")
        self._bind_setting_widget(builder.get_object("settings_name_does_not_contain"), "name_does_not_contain")
        
        self._window.set_default_size(self._application.get_setting("window_width"), self._application.get_setting("window_height"))
        if self._application.get_setting("window_maximized"):
            self._window.maximize()
        
        if self._application.get_setting("sidebar_width") >= 0:
            builder.get_object("main_paned").set_position(self._application.get_setting("sidebar_width"))
        builder.get_object("sidebar_notebook").connect("size-allocate", self._on_sidebar_notebook_size_allocate, builder.get_object("main_paned"))
        
        sidebar_page = self._application.get_setting("sidebar_active_page")
        if sidebar_page >= 0 and sidebar_page < builder.get_object("sidebar_notebook").get_n_pages():
            builder.get_object("sidebar_notebook").set_current_page(sidebar_page)
        builder.get_object("sidebar_notebook").connect("switch-page", self._on_sidebar_notebook_switch_page)
        
        builder.get_object("min_size_box").set_sensitive(self._application.get_setting("min_size_enabled"))
        builder.get_object("max_size_box").set_sensitive(self._application.get_setting("max_size_enabled"))
        builder.get_object("settings_min_size_enabled").connect("toggled", lambda w,b: b.set_sensitive(w.get_active()), builder.get_object("min_size_box"))
        builder.get_object("settings_max_size_enabled").connect("toggled", lambda w,b: b.set_sensitive(w.get_active()), builder.get_object("max_size_box"))
        
        self._search_entry = builder.get_object("search_entry")
        self._search_entry.set_text(self._application.get_setting("search_pattern"))
        
        self._search_entry.connect("activate", lambda e,a: a.activate(), builder.get_object("search_action"))
        builder.get_object("search_action").connect("activate", lambda w: self._run_search())
        self._stop_search_action = builder.get_object("stop_search_action")
        self._stop_search_action.connect("activate", lambda w: self._stop_search())
        
        self._results_list = ResultsList(self._application)
        builder.get_object("results_list_container").add(self._results_list)
        self._results_list.connect_event("selection_changed", self._on_results_list_selection_changed, builder)
        
        self._plugins_updater_dialog = builder.get_object("plugins_updater_dialog")
        self._plugins_updater_progressbar = builder.get_object("plugins_updater_progressbar")
        self._plugins_updater_dialog.connect("delete_event", lambda w,e: True)
        
        self._main_progressbar = builder.get_object("main_progressbar")
        
        self._preferences_plugins_list = PreferencesPluginsList(self._application.search_engine)
        builder.get_object("preferences_plugins_list_container").add(self._preferences_plugins_list)
        
        self._window.connect("map_event", self._on_window_map_event, builder)
        
        self._build_about_dialog(builder)
    
    def _build_about_dialog(self, builder):
        builder.get_object("about_dialog").set_program_name(APPNAME)
        builder.get_object("about_dialog").set_version(VERSION)
        builder.get_object("about_dialog").set_authors([(a["name"] + "<" + a["email"] + ">") for a in AUTHORS])
        builder.get_object("about_dialog").set_comments(APP_DESCRIPTION)
        builder.get_object("about_dialog").set_license(LICENSE)
    
    def _on_results_list_selection_changed(self, results_list, selection, builder):
        if len(selection) > 0:
            element = selection[0]
            builder.get_object("torrent_details_title").set_text(element.title)
        else:
            builder.get_object("torrent_details_title").set_text("")
    
    def _on_window_map_event(self, window, event, builder):
        if not self._window_mapped_once:
            self._window_mapped_once = True
            self._window.connect("window-state-event", self._on_window_state_event)
            self._trigger_event("ready")
    
    def _run_search(self):
        pattern = self._search_entry.get_text()
        if pattern != "":
            self._application.run_search(pattern)
    
    def _stop_search(self):
        self._application.stop_search()
    
    def _on_sidebar_notebook_switch_page(self, notebook, page, page_num):
        self._application.set_setting("sidebar_active_page", page_num)
    
    def _on_sidebar_notebook_size_allocate(self, notebook, size_allocation, paned):
        self._application.set_setting("sidebar_width", paned.get_position())
    
    def _get_window_maximized(self):
        return ((self._window.get_window().get_state() & Gdk.WindowState.MAXIMIZED) == Gdk.WindowState.MAXIMIZED)
    
    def _on_window_state_event(self, window, event):
        self._application.set_setting("window_maximized", self._get_window_maximized())
    
    def _bind_setting_widget(self, widget, setting_key):
        setting = self._application.settings_manager.get_setting(setting_key)
        if setting != None:
            if setting.get_type() == "bool" and type(widget) in [Gtk.CheckButton, Gtk.Switch]:
                widget.set_active(setting.get_value())
                if type(widget) == Gtk.CheckButton:
                    widget.connect("toggled", lambda w,s: s.set_value(w.get_active()), setting)
                else:
                    widget.connect("notify::active", lambda w,i,s: s.set_value(w.get_active()), setting)
            elif setting.get_type() == "str" and hasattr(widget, "get_text") and hasattr(widget, "set_text"):
                widget.set_text(setting.get_value())
                widget.connect("changed", lambda w,s: s.set_value(w.get_text()), setting)
            elif setting.get_type() in ["str_choice"] and type(widget) == Gtk.ComboBox:
                if setting.get_type() == "str_choice":
                    model = Gtk.ListStore(str, str)
                widget.set_model(model)
                r = Gtk.CellRendererText()
                widget.pack_start(r, True)
                widget.add_attribute(r, 'text', 1)
                active_index = None
                for i in range(len(setting.values)):
                    value = setting.values[i]
                    model.append((value['key'], value['label']))
                    if value['key'] == setting.get_value():
                        active_index = i
                if active_index != None:
                    widget.set_active(active_index)
                widget.connect("changed",  lambda w,s: s.set_value(s.values[w.get_active()]["key"]), setting)
            elif setting.get_type() == "int" and type(widget) == Gtk.SpinButton:
                widget.set_value(setting.get_value())
                widget.connect("value-changed", lambda w,s: s.set_value(w.get_value()), setting)
            else:
                logging.warn("Impossible to bind widget with setting key \"%s\"" % setting_key)
    
    def _on_plugins_updater_finished(self, updater):
        self._plugins_updater_dialog.response(Gtk.ResponseType.OK)
    
    def _on_plugins_updater_progress(self, updater):
        self._plugins_updater_progressbar.set_fraction(updater.progress)
    
    def _on_plugins_updater_pulse(self, updater):
        self._plugins_updater_progressbar.pulse()
    
    def mainloop(self):
        self._application.plugins_updater.connect_event("finished", self._on_plugins_updater_finished)
        self._application.plugins_updater.connect_event("progress", self._on_plugins_updater_progress)
        self._application.plugins_updater.connect_event("pulse", self._on_plugins_updater_pulse)
        self._plugins_updater_dialog.show_all()
        while self._plugins_updater_dialog.run() != Gtk.ResponseType.OK:
            pass
        self._plugins_updater_dialog.hide()
        Gtk.main()
    
    def stop_mainloop(self):
        Gtk.main_quit()
    
    def clear_results(self):
        self._results_list.clear()
    
    def show_search_progress(self, progress):
        self._main_progressbar.set_fraction(progress)
    
    def display_result(self, result):
        self._results_list.add_result(result)
