# main.py
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# Copyright © 2012 Gwendal Le Bihan
# 
# This file is part of Torrent Search.
# 
# Torrent Search is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Torrent Search is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Torrent Search.  If not, see <http://www.gnu.org/licenses/>.

import threading
from .HTTPManager import HTTPManager
from .SettingsManager import SettingsManager
from .SearchEngine import SearchEngine
from .PluginsUpdater import PluginsUpdater
from .EventsObject import EventsObject
from . import env
from . import frontends

class TorrentSearch(EventsObject):
    def __init__(self, cli_args):
        EventsObject.__init__(self)
        
        self.http_manager = HTTPManager(self)
        self.settings_manager = SettingsManager(cli_args)
        self.search_engine = SearchEngine(self, self.get_setting("plugins_path"))
        self.search_engine.connect_event("result_added", self._on_search_result_added)
        self.plugins_updater = PluginsUpdater(self)
        self.plugins_updater.connect_event("finished", self._on_plugins_updater_finished)
        
        self._frontend = frontends.load_frontend(self.get_setting("frontend"), self, cli_args.share_dir)
        if self._frontend == None:
            raise Exception(_("No frontend available !"))
        
        self._current_search_pattern = ""
    
    def should_show_result(self, search_result):
        if self.get_setting("hide_no_seeders") and search_result.seeders <= 0:
            return False
            
        pattern = self.get_setting("name_contains")
        if pattern != "" and not pattern.lower() in search_result.title.lower():
            return False
            
        pattern = self.get_setting("name_does_not_contain")
        if pattern != "" and pattern.lower() in search_result.title.lower():
            return False
        
        if self.get_setting("exact_phrase") and not self._current_search_pattern.lower() in search_result.title.lower():
            return False
        
        if self.get_setting("all_words"):
            for word in self._current_search_pattern.split(" "):
                if not word in search_result.title.lower():
                    return False
        
        if self.get_setting("min_size_enabled"):
            min_size = self.get_setting("min_size")
            min_size_unit = self.get_setting("min_size_unit")
            units = ["B", "KB", "MB", "GB", "TB"]
            ui = units.index(min_size_unit)
            while ui > 0:
                min_size *= 1024
                ui -= 1
            if search_result.size < min_size:
                return False
            
        if self.get_setting("max_size_enabled"):
            max_size = self.get_setting("max_size")
            max_size_unit = self.get_setting("max_size_unit")
            units = ["B", "KB", "MB", "GB", "TB"]
            ui = units.index(max_size_unit)
            while ui > 0:
                max_size *= 1024
                ui -= 1
            if search_result.size > max_size:
                return False
            
        return True
        
    def _on_search_result_added(self, search_engine, plugin, result):
        self._search_results.append(result)
        self._frontend.display_result(result)
        if result.need_additional_infos:
            result.fetch_additional_infos()
    
    def _on_plugins_updater_finished(self, updater):
        self._check_new_version()
    
    def get_setting(self, key):
        return self.settings_manager.get_setting_value(key)
        
    def set_setting(self, key, value):
        self.settings_manager.set_setting_value(key, value)
    
    def timeout_add(self, delay, callback, *args):
        self._frontend.timeout_add(delay, callback, *args)
    
    def wait_for_thread(self, thread, callback, *args, **kwargs):
        self._frontend.timeout_add(100, self._check_thread_ended, thread, callback, args, kwargs)
    
    def _check_thread_ended(self, thread, callback, args, kwargs):
        if thread.isAlive():
            return True
        else:
            callback(thread, *args, **kwargs)
            return False
    
    def _check_new_version(self):
        pass#self._http_manager.request_async("http://admin.gwendallebihan.net/", callback = self._on_version_data_received)
    
    def _on_version_data_received(self, response):
        print(response)
    
    def run_search(self, pattern):
        self.stop_search()
        self._search_results = []
        self._frontend.clear_results()
        self._current_search_pattern = pattern
        self.search_engine.run(pattern)
    
    def stop_search(self):
        self.search_engine.stop()
        
    def run(self):
        self._frontend.init()
        self.search_engine.load_plugins()
        self.plugins_updater.run()
        if self.get_setting("search_pattern") != "":
            self._frontend.connect_event("ready", lambda f,p: self.run_search(p), self.get_setting("search_pattern"))
        self._frontend.mainloop()
        self.stop_search()
        self.http_manager.stop()
